@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2002 Hewlett-Packard Co.
+ *  Copyright (C) 2001-2003 Hewlett-Packard Co.
  *	Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
  * This file is part of the ELILO, the EFI Linux boot loader.
@@ -843,7 +843,7 @@ get_config_file(VOID)
 }
 
 EFI_STATUS
-read_config(CHAR16 *filename)
+read_config(CHAR16 *filename, INTN retry)
 {
 	EFI_STATUS status;
 	INTN ret;
@@ -856,8 +856,12 @@ read_config(CHAR16 *filename)
 
 	status = fops_open(filename, &config_fd);
 	if (EFI_ERROR(status)) {
-		if (elilo_opt.parse_only) {
-			VERB_PRT(3, Print(L"config file %s not found\n", filename));
+		/* 
+		 * if the user explicitely specified a filename and we can't
+		 * find it, then we must fail.
+		 */
+		if (elilo_opt.parse_only || retry == 0) {
+			VERB_PRT(3, Print(L"cannot open config file %s\n", filename));
 			return status;
 		}
 		/*
@@ -913,7 +917,7 @@ read_config(CHAR16 *filename)
 
 	return EFI_SUCCESS;
 }
-	
+
 VOID
 print_label_list(VOID)
 {
@@ -924,7 +928,6 @@ print_label_list(VOID)
 	for (img = image_list; img; img = img->next) {
 		if (img != dfl) Print(L"%s   ", img->label);
 	}
-	Print(L"(or any kernel file name: [dev_name:]/path/file)\n");
 }
 
 /* Make labels and corresponding descriptions available to choosers.  The
