@@ -1,6 +1,7 @@
-/* elilo.c - IA-64 Linux loader
+/* 
+ * elilo.c - IA-64/IA-32 EFI Linux loader
  *
- *  Copyright (C) 1999-2002 Hewlett-Packard Co.
+ *  Copyright (C) 1999-2003 Hewlett-Packard Co.
  *	Contributed by David Mosberger <davidm@hpl.hp.com>.
  *	Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
@@ -39,7 +40,7 @@
 #include "loader.h"
 #include "config.h" /* for config_init() */
 
-#define ELILO_VERSION			L"3.2"
+#define ELILO_VERSION			L"3.4"
 #define ELILO_SHARED_CMDLINE_OPTS	L"pPMC:aDhd:i:vVc:E"
 
 elilo_config_t elilo_opt;
@@ -188,7 +189,7 @@ do_launch:
 	/* No console output permitted after create_boot_params()! */
 	if ((bp=create_boot_params(cmdline, &imem, &cookie)) == 0) goto error;
 
-	/* terminate bootservices: */
+	/* terminate bootservices */
 	status = BS->ExitBootServices(image, cookie);
 	if (EFI_ERROR(status)) goto bad_exit;
 
@@ -239,6 +240,10 @@ elilo_help(VOID)
  * from network. In this case, it looks like LoadOptions/LoadOptionsSize contain
  * garbage
  */
+static CHAR16 *default_load_options;
+static UINTN default_load_options_size;
+static INTN done_fixups;
+
 static VOID
 fixupargs(EFI_LOADED_IMAGE *info)
 {
@@ -250,10 +255,29 @@ fixupargs(EFI_LOADED_IMAGE *info)
 	status = BS->HandleProtocol (info->DeviceHandle, &PxeBaseCodeProtocol, (VOID **)&pxe);
 	if (EFI_ERROR(status)) return;
 
-	info->LoadOptions    = FAKE_ELILONAME;
+	default_load_options      = info->LoadOptions;
+	default_load_options_size = info->LoadOptionsSize;
+
+	info->LoadOptions     = FAKE_ELILONAME;
 	info->LoadOptionsSize = StrLen(info->LoadOptions)*sizeof(CHAR16);
+
+	done_fixups = 1;
 }
 
+/*
+ * we restore the arguments in case we modified them just to make sure
+ * we don't confuse caller.
+ */
+static VOID
+unfixupargs(EFI_LOADED_IMAGE *info)
+{
+	if (done_fixups == 0) return;
+
+	info->LoadOptions     = default_load_options;
+	info->LoadOptionsSize = default_load_options_size;
+}
+
+
 /*
  * in order to get fully detailed EFI path names to devices, EDD3.0 specification must
  * be turned on. On new versions of EFI, this is the default. An environment variable
@@ -338,9 +362,14 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	EFI_LOADED_IMAGE *info;
 	EFI_STATUS status, ret = EFI_LOAD_ERROR;
 	INTN argc = 0, c;
-	INTN edd30_status;
+	INTN edd30_status, retry;
 	CHAR16 *ptr, *arglist = NULL;
 	BOOLEAN devices_initialized = FALSE;
+	CHAR16 dpath[FILENAME_MAXLEN];
+	CHAR16 *devpath;
+
+	//elilo_opt.verbose=3;
+	//elilo_opt.debug=1;
 
 	/* initialize global variable */
 	systab = system_tab;
@@ -392,7 +421,6 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	ERR_PRT((L"LoadOptions=%x OpenSize=%d", info->LoadOptions, info->LoadOptionsSize));
 	 */
 
-
 	/*
 	 * in case there is something wrong with the default boot_device
 	 * we default to basic fixups and we need to force interactive
@@ -400,12 +428,20 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	 */
 	fixupargs(info);
 
+#if 0
+	Print(L"LoadOptions=%x OpenSize=%d\n", info->LoadOptions, info->LoadOptionsSize);
+	{ INTN i; for (i=0; i< info->LoadOptionsSize>>1; i++) Print(L"options[%d]=%d (%c)\n", i, ((CHAR16 *)info->LoadOptions)[i], ((CHAR16 *)info->LoadOptions)[i]); }
+#endif
+
 	/*
 	 * we must copy argument because argify modifies the string.
 	 * This caused problems when arguments are coming from NVRAM
 	 * as passed by the EFI boot manager
+	 *
+	 * We add an extra character to the buffer in case the LoadOptions is not 
+	 * NULL terminated. The extra space will be used to ADD the extra terminator.
 	 */
-	arglist = alloc(info->LoadOptionsSize, EfiLoaderData);
+	arglist = alloc(info->LoadOptionsSize+sizeof(CHAR16), EfiLoaderData);
 	if (arglist == NULL) {
 		ERR_PRT((L"cannot copy argument list"));
 		return EFI_OUT_OF_RESOURCES;
@@ -506,10 +542,13 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	if (init_devices(info->DeviceHandle) == -1) goto do_exit;
 
 	devices_initialized = TRUE;
+
+	devpath = DevicePathToStr(info->FilePath);
+
 	/*
 	 * set per fileops defaults files for configuration and kernel
 	 */
-	fops_setdefaults(elilo_opt.default_config, elilo_opt.default_kernel, FILENAME_MAXLEN);
+	fops_setdefaults(elilo_opt.default_config, elilo_opt.default_kernel, FILENAME_MAXLEN, devpath);
 
 	/*
 	 * XXX: won't be visible if verbose not required from command line
@@ -519,20 +558,34 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	/*
 	 * use default config file if not specified by user
 	 */
-	ptr = elilo_opt.config[0] == CHAR_NULL ?  elilo_opt.default_config : elilo_opt.config;
+	ptr = elilo_opt.config[0] == CHAR_NULL ?  (retry=1,elilo_opt.default_config) : (retry=0,elilo_opt.config);
 
 	/*
 	 * parse config file (verbose becomes visible if set)
 	 */
-	ret = read_config(ptr);
-
+	ret = read_config(ptr, retry);
+	Print(L"read_config=%r\n", ret);
+	/*
+	 * when the config file is not found, we fail only if:
+	 * 	- the user did not specified interactive mode
+	 * 	- the user did not explicitely specify the config file
+	 */
+	if (ret == EFI_NOT_FOUND || ret == EFI_TFTP_ERROR) {
+		if (elilo_opt.prompt == 0 && elilo_opt.config[0] != CHAR_NULL) {
+			Print(L"config file %s not found\n", ptr);
+			goto do_exit;
+		}
+		fops_getdefault_path(dpath, FILENAME_MAXLEN);
+		if (ret == EFI_TFTP_ERROR)
+			Print(L"no config file found on TFTP server in %s\n", dpath);
+		else
+			Print(L"no config file found in %s\n", dpath);
+	}
 	/*
 	 * stop if just doing parsing
 	 */
 	if (elilo_opt.parse_only) {
-		if (ret == EFI_NOT_FOUND) 
-			Print(L"config file %s not found\n", ptr);
-		else if (ret == EFI_SUCCESS)
+		if (ret == EFI_SUCCESS)
 			Print(L"Config file %s parsed successfully\n", ptr);
 		goto do_exit;
 	}
@@ -541,11 +594,10 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	 * we force interactive mode to give a chance to the user.
 	 * We also clear the error. 
 	 */
-	if (ret == EFI_INVALID_PARAMETER) {
+	if (ret && argc == 1) {
 		Print(L"forcing interactive mode because of errors\n");
 		elilo_opt.prompt = 1;
 	}
-	ret = EFI_SUCCESS;
 
 	/*
 	 * If EDD30 EFI variable was not set to TRUE (or not defined), we
@@ -559,21 +611,13 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 		force_edd30();
 	}
 
-
-
-
-	/* 
-	 * if something went wrong with config file 
-	 * and no argument was given, then force interactive mode
-	 */
-	if (argc == 1 && ret == EFI_INVALID_PARAMETER) {
-		elilo_opt.prompt = 1;
-	}
+	ret = EFI_LOAD_ERROR;
 
 	/*
 	 * if the user specified a kernel on the command line
 	 * then we don't go to interactive mode even if it
-	 * was set in the config file
+	 * was set in the config file or set because of an
+	 * error parsing the config file.
 	 */
 	if (argc > Optind) elilo_opt.prompt = 0;
 
@@ -596,7 +640,9 @@ efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *system_tab)
 	main_loop(info->DeviceHandle, argv, argc, Optind, image);
 	/* should not return */
 do_exit:
-	if (arglist) free(arglist);
+	unfixupargs(info);
+
+	//if (arglist) free(arglist);
 
 	/* free all resources assiocated with file accesses */
 	if (devices_initialized) close_devices();
