@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2002 Hewlett-Packard Co.
+ *  Copyright (C) 2001-2003 Hewlett-Packard Co.
  *	Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *	Contributed by Mike Johnston <johnston@intel.com>
  *	Contributed by Chris Ahna <christopher.j.ahna@intel.com>
@@ -39,6 +39,11 @@
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+/* extern loader_ops_t plain_loader, gzip_loader; */
+
+efi_ia32_boot_params_t efi_ia32_bp;
+
+
 /*
  * Descriptor table base addresses & limits for Linux startup.
  */
@@ -72,34 +77,6 @@ UINT16 init_gdt[] = {
 
 UINTN sizeof_init_gdt = sizeof init_gdt;
 
-/*
- *
- */
-
-_8259_t _8259[] = {
-	{ 0x20, 0x11 },
-	{ 0xA0, 0x11 },
-	{ 0x21, 0x20 },
-	{ 0xA1, 0x28 },
-	{ 0x21, 0x04 },
-	{ 0xA1, 0x02 },
-	{ 0x21, 0x01 },
-	{ 0xA1, 0x01 },
-	{ 0xA1, 0xFF },
-	{ 0x21, 0xFB },
-};
-
-UINTN _8259_entries = (sizeof _8259) / sizeof _8259[0];
-
-/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-/*
- * DeviceIoProtocol interface.
- * This interface will be used to program the 8259.
- */
-
-EFI_DEVICE_IO_INTERFACE *efi_devio_intf = NULL;
 
 /*
  * Highest available base memory address.
@@ -157,14 +134,14 @@ BOOLEAN can_reloc_boot_params = FALSE;
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static INTN
-probe_x86_boot(CHAR16 *kname)
+probe_bzImage_boot(CHAR16 *kname)
 {
 	EFI_STATUS efi_status;
 	UINTN size;
 	fops_fd_t fd;
 	UINT8 bootsect[512];
 
-	DBG_PRT((L"probe_x86_boot()\n"));
+	DBG_PRT((L"probe_bzImage_boot()\n"));
 
 	if (!kname) {
 		ERR_PRT((L"kname == %xh", kname));
@@ -207,7 +184,7 @@ probe_x86_boot(CHAR16 *kname)
 	 */
 
 	if (bootsect[0x1FE] != 0x55 || bootsect[0x1FF] != 0xAA) {
-		ERR_PRT((L"%s is not an x86 boot kernel image.\n", kname));
+		ERR_PRT((L"%s is not a bzImage kernel image.\n", kname));
 		fops_close(fd);
 		free_kmem();
 		return -1;
@@ -221,7 +198,7 @@ probe_x86_boot(CHAR16 *kname)
 	DBG_PRT((L"bootsect[1F1h] == %d setup sectors\n", bootsect[0x1F1]));
 
 	if (bootsect[0x1F1] < 1 || bootsect[0x1F1] > 64) {
-		ERR_PRT((L"%s is not a valid x86 boot kernel image.",
+		ERR_PRT((L"%s is not a valid bzImage kernel image.",
 			kname));
 
 		fops_close(fd);
@@ -286,6 +263,17 @@ probe_x86_boot(CHAR16 *kname)
 		return -1;
 	}
 
+	/*
+	 * Allocate memory for kernel.
+	 */
+
+	if (alloc_kmem(kernel_start, EFI_SIZE_TO_PAGES(kernel_size))) {
+		ERR_PRT((L"Could not allocate kernel memory."));
+		return -1;
+	} else {
+		VERB_PRT(3, Print(L"kernel_start: 0x%x  kernel_size: %d\n", kernel_start, kernel_size));
+	}
+
 	/*
 	 * Now read the rest of the kernel image into memory.
 	 */
@@ -318,9 +306,9 @@ probe_x86_boot(CHAR16 *kname)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static INTN
-load_x86_boot(CHAR16 *kname, kdesc_t *kd)
+load_bzImage_boot(CHAR16 *kname, kdesc_t *kd)
 {
-	DBG_PRT((L"load_x86_boot()\n"));
+	DBG_PRT((L"load_bzImage_boot()\n"));
 
 	if (!kname || !kd) {
 		ERR_PRT((L"kname=0x%x  kd=0x%x", kname, kd));
@@ -341,11 +329,11 @@ load_x86_boot(CHAR16 *kname, kdesc_t *kd)
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-static loader_ops_t loader_x86_boot = {
+static loader_ops_t loader_bzImage_boot = {
 	NULL,
-	L"loader_x86_boot",
-	&probe_x86_boot,
-	&load_x86_boot
+	L"loader_bzImage_boot",
+	&probe_bzImage_boot,
+	&load_bzImage_boot
 };
 
 /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
@@ -353,84 +341,17 @@ static loader_ops_t loader_x86_boot = {
 INTN
 sysdeps_init(EFI_HANDLE dev)
 {
-	EFI_STATUS efi_status;
-	EFI_HANDLE *devio_handle_list;
-	UINTN devio_handle_cnt;
 
 	DBG_PRT((L"sysdeps_init()\n"));
 
-	/*
-	 * Locate handle(s) w/ DeviceIoProtocol.
-	 */
-
-	DBG_PRT((L"LibLocateHandle()  "));
-
-	efi_status = LibLocateHandle(
-		ByProtocol,
-		&DeviceIoProtocol,
-		NULL,
-		&devio_handle_cnt,
-		&devio_handle_list);
-
-	DBG_PRT((L"%xh (%r)\n", efi_status, efi_status));
-	DBG_PRT((L"devio_handle_cnt: %d  devio_handle_list: %xh\n", devio_handle_cnt, devio_handle_list));
-
-	if (EFI_ERROR(efi_status)) {
-		ERR_PRT((L"Could not locate DeviceIoProtocol handle."));
-		return -1;
-	}
-
-	if (!devio_handle_cnt || !devio_handle_list) {
-		ERR_PRT((L"Could not locate DeviceIoProtocol handle."));
-		return -1;
-	}
-
-	/*
-	 * Locate DeviceIoProtocol interface.
-	 */
-
-	DBG_PRT((L"BS->HandleProtocol()  "));
-
-        efi_status = BS->HandleProtocol( devio_handle_list[0], &DeviceIoProtocol,
-					(VOID **)&efi_devio_intf);
-
-	DBG_PRT((L"%xh (%r)\n", efi_status, efi_status));
-
-	if (EFI_ERROR(efi_status)) {
-		ERR_PRT((L"Could not locate deviceIoProtocol interface."));
-		FreePool(devio_handle_list);
-		return -1;
-	}
-
-	if (!efi_devio_intf) {
-		ERR_PRT((L"Could not locate deviceIoProtocol interface."));
-		FreePool(devio_handle_list);
-		return -1;
-	}
-
 	/*
 	 * Register our loader(s)...
 	 */
 
-	loader_register(&loader_x86_boot);
+	loader_register(&loader_bzImage_boot);
+	/* loader_register(&plain_loader); */	
+	/* loader_register(&gzip_loader); */
 
-	/*
-	 * Allocate memory for kernel.
-	 */
-
-	if (alloc_kmem(kernel_start, EFI_SIZE_TO_PAGES(kernel_size))) {
-		ERR_PRT((L"Could not allocate kernel memory."));
-		FreePool(devio_handle_list);
-		return -1;
-	} else {
-		VERB_PRT(3, Print(L"kernel_start: 0x%x  kernel_size: %d\n", kernel_start, kernel_size));
-	}
-
-	/*
-	 * Clean up and return.
-	 */
-
-	FreePool(devio_handle_list);
 
 	return 0;
 }
@@ -844,6 +765,34 @@ sysdeps_create_boot_params(
 	}
 
 	*cookie = mdesc.cookie;
+	bp->s.efi_mem_map = (UINTN)mdesc.md;
+	bp->s.efi_mem_map_size = mdesc.map_size;
+	bp->s.efi_mem_desc_size = mdesc.desc_size;
+	bp->s.efi_mem_desc_ver = mdesc.desc_version;
+	bp->s.efi_sys_tbl = (UINTN)systab;
+	
+	/*
+	 * my_ia32_boot_params and get ready to slap them into 0x00104c00
+	 */
+
+	efi_ia32_bp.size= sizeof(efi_ia32_bp);
+	efi_ia32_bp.command_line = (UINT32) cmdline;
+	efi_ia32_bp.efi_sys_tbl = bp->s.efi_sys_tbl;
+	efi_ia32_bp.efi_mem_map = bp->s.efi_mem_map;
+	efi_ia32_bp.efi_mem_map_size = bp->s.efi_mem_map_size;
+	efi_ia32_bp.efi_mem_desc_size = bp->s.efi_mem_desc_size;
+	efi_ia32_bp.efi_mem_desc_version = bp->s.efi_mem_desc_ver;
+	efi_ia32_bp.initrd_start = (UINTN)initrd->start_addr;
+	efi_ia32_bp.initrd_size = initrd->pgcnt * EFI_PAGE_SIZE;
+	efi_ia32_bp.loader_start = 0;
+	efi_ia32_bp.loader_size = 0;
+	efi_ia32_bp.kernel_start = bp->s.kernel_start;
+	efi_ia32_bp.kernel_size = kernel_size;
+	efi_ia32_bp.num_cols = cols;
+	efi_ia32_bp.num_rows = rows;
+	efi_ia32_bp.orig_x = col;
+	efi_ia32_bp.orig_y = row;
+
 
 	return 0;
 }
